package com.ai.ipu.cache;

import java.util.HashMap;
import java.util.Map;

import com.ai.ipu.basic.log.ILogger;
import com.ai.ipu.basic.log.IpuLoggerFactory;
import com.ai.ipu.cache.config.IpuCacheConfig;
import com.ai.ipu.cache.mem.impl.WadeMemCache;
import com.ai.ipu.cache.redis.RedisCacheFactory;
import com.ai.ipu.cache.redis.impl.JedisCache;
import com.ai.ipu.cache.redis.impl.JedisClusterCache;
import com.ai.ipu.cache.util.IpuCacheConstant;
import com.ailk.cache.memcache.MemCacheFactory;

/**
 * 缓存工厂方法
 */
public class CacheFactory {
    private static final ILogger LOGGER = IpuLoggerFactory.createLogger(CacheFactory.class);
    private static Map<String, ICache> caches = new HashMap<String, ICache>();
    public static final String DEFAULT_CACHE_NAME = "redis";
    
    private CacheFactory() {
    }

    public enum CacheType {
        mem, redis, jvm, wadeMem;
    }

    /**
     * 根据ipu-cache.xml或memcache.xml中的配置信息构造缓存实例
     * @param type 缓存的类型
     * @param cacheName 缓存的名称
     * @return 缓存的实例
     * @throws Exception
     */
    @SuppressWarnings("unused")
    public static ICache getCache(CacheType type, String cacheName) throws Exception {
        ICache cache = caches.get(cacheName);

        if (null == cache) { // 首次使用时加载缓存
            cache = getCache(cache, type, cacheName);
        }
        return cache;
    }

    /**
     * 根据ipu-cache.xml中的配置信息构造缓存实例，
     * @param cacheName 缓存的名称
     * @return 缓存的实例
     * @throws Exception
     */
    public static ICache getCache(String cacheName) throws Exception {
        String cacheType = IpuCacheConfig.getCacheType(cacheName);
        cacheType = cacheType == null ? DEFAULT_CACHE_NAME : cacheType;
        return getCache(CacheType.valueOf(cacheType), cacheName);
    }
    
    private static ICache createCache(CacheType type, String cacheName) throws Exception {
        /* 根据cacheName获取缓存类型 */
        switch (type) {
        case mem:
            break;
        case redis:
            if (IpuCacheConstant.ClientType.JEDIS_CLIENT.equalsIgnoreCase(IpuCacheConfig.getCacheDefaultAttr(cacheName,
                    IpuCacheConstant.Redis.CLIENT_TYPE, IpuCacheConstant.ClientType.JEDIS_CLIENT)))
                return new JedisCache(RedisCacheFactory.getJedis(cacheName));
            else
                return new JedisClusterCache(RedisCacheFactory.getJedisCluster(cacheName));
        case jvm:
            
            break;
        case wadeMem:
            return new WadeMemCache(MemCacheFactory.getCache(cacheName));
        default:
            break;
        }
        return null;
    }
    
    /**
     * 判断是否IPU缓存实例是否存在
     * @return {@code true} 存在; {@code false} 不存在
     */
    public static boolean isIpuCache(String cacheName) throws Exception {
        Map<String, Object> cacheEntity = IpuCacheConfig.getCacheEntity(cacheName);
        return cacheEntity != null && !cacheEntity.isEmpty();
    }
    
    /**
     * redis使用后，需要手工释放，进行资源回收
     * 集群不需要释放
     * 
     * @param cacheName
     * @param cache redis实例
     * @throws Exception
     */
    public static void close(String cacheName, ICache cache) throws Exception {
        if (cache == null)
            return;
        if (cache instanceof JedisCache)
            ((JedisCache)cache).close();
        else if (caches!=null && !caches.isEmpty())
            caches.remove(cacheName);
    }
    
    private static ICache getCache(ICache iCache, CacheType type, String cacheName) throws Exception {
        ICache cache = iCache;
        synchronized (CacheFactory.class) {
            if (cache!=null){ //避免时间差
                return cache;
            }
            cache = createCache(type, cacheName);
            if(!(type.equals(CacheType.redis)&&IpuCacheConstant.ClientType.JEDIS_CLIENT.equalsIgnoreCase(IpuCacheConfig.getCacheDefaultAttr(cacheName,
                    IpuCacheConstant.Redis.CLIENT_TYPE, IpuCacheConstant.ClientType.JEDIS_CLIENT))))
            {
                caches.put(cacheName, cache);
                LOGGER.info("初始化缓存"+cacheName+"成功");
            }
        }
        return cache;
    }
}
