package com.ai.ipu.cache.redis.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import redis.clients.jedis.JedisCluster;

import com.ai.ipu.basic.string.StringUtil;
import com.ai.ipu.basic.util.serial.DefaultSerializable;
import com.ai.ipu.basic.util.serial.ISerializable;
import com.ai.ipu.cache.redis.IRedisCache;
import com.ai.ipu.cache.redis.listener.AbstractPubSubListener;
import com.ailk.org.apache.commons.lang3.ArrayUtils;

/**
 * redis缓存的实现
 */
public class JedisClusterCache implements IRedisCache {
    private JedisCluster redisCache;
    /**
     * 对象序列化与反序列化接口。
     */
    private static final ISerializable SERIALIZER = new DefaultSerializable();
    
    public JedisClusterCache(JedisCluster jedisCluster)
    {
        this.redisCache = jedisCluster;
    }

    /**
     * 存
     */
    @Override
    public boolean put(Object key, Object value) throws Exception {
        String result = "";
        if (key instanceof String)
            result = redisCache.set(key.toString().getBytes(), SERIALIZER.encode(value));
        else if (key instanceof byte[])
            result = redisCache.set((byte[])key, (byte[])value);
        
        return "OK".equalsIgnoreCase(result);
    }

    /**
     * 取
     */
    @Override
    public Object get(Object key) throws Exception {
        if (key instanceof String)
        {
            if (StringUtil.isEmpty((String)key))
                return null;
            return SERIALIZER.decode(redisCache.get(key.toString().getBytes()));
        }
        else if (key instanceof byte[])
            return redisCache.get((byte[])key);
        return null;
    }

    /**
     * 移除
     */
    @Override
    public boolean remove(Object key) throws Exception {
        long result = -1;
        if (key instanceof String)
            result = redisCache.del((String)key);
        else if (key instanceof byte[])
            result = redisCache.del((byte[])key);
        return result == 1?true:false;
    }

    /**
     * 清除
     */
    @Override
    public void clear() throws Exception {
        //集群不支持clear
        throw new Exception("集群不支持clear");
    }

    /**
     * key是否存在
     */
    @Override
    public boolean keyExists(String cacheKey) {
        return redisCache.exists(cacheKey);
    }

    /**
     *  自增
     * @param key
     * @return
     */
    @Override
    public Long incr(String key) {
        return redisCache.incr(key);
    }

    /**
     * 自定义步长自增
     * @param key
     * @param step
     * @return
     */
    @Override
    public Long incrBy(String key, long step) {
        return redisCache.incrBy(key, step);
    }

    /**
     * 设置失效时间
     * @param key
     * @param timeoutSeconds
     * @return
     */
    @Override
    public Long expire(String key, int timeoutSeconds) {
        return redisCache.expire(key, timeoutSeconds);
    }

    /**
     * 设置失效时间点
     * @param key
     * @param millisSeconds
     * @return
     */
    @Override
    public Long expireAt(String key, long millisSeconds) {
        return redisCache.expireAt(key, millisSeconds);
    }

    /**
     * 设置数据
     * @param key
     * @param value
     * @param timeoutSeconds
     * @return
     * @throws Exception
     */
    @Override
    public boolean put(Object key, Object value, int timeoutSeconds) throws Exception {
        String result = "";
        if (key instanceof String)
            result = redisCache.setex(key.toString().getBytes(), timeoutSeconds, SERIALIZER.encode(value));
        else if (key instanceof byte[])
            result = redisCache.setex((byte[])key, timeoutSeconds, (byte[])value);
        
        return "OK".equalsIgnoreCase(result);
    }

    /**
     * 设置map
     * @param key
     * @param map
     * @return
     */
    @Override
    public boolean putMap(String key, Map<String, String> map) {
        String result = redisCache.hmset(key, map);
        return "OK".equalsIgnoreCase(result);
    }

    /**
     * 获取map的长度
     * @param key
     * @return
     */
    @Override
    public long getMapLens(String key) {
        return redisCache.hlen(key);
    }

    /**
     * 获取map中所有的key
     * @param key
     * @return
     */
    @Override
    public Set<String> getMapKeys(String key) {
        Set<String> result = new TreeSet<String>(new MyComparator());
        result.addAll(redisCache.hkeys(key));
        return result;
    }

    /**
     * 获取map中所有的value
     * @param key
     * @return
     */
    @Override
    public List<String> getMapVals(String key) {
        return redisCache.hvals(key);
    }

    /**
     * 获取map中的value
     * @param key
     * @param fields
     * @return
     */
    @Override
    public List<String> takeMapVals(String key, String... fields) {
        if (keyExists(key))
            return redisCache.hmget(key, fields);
        else
            return new ArrayList<String>();
    }

    /**
     * 删除map中的value
     * @param key
     * @param elements
     * @return
     */
    @Override
    public boolean delMapElement(String key, String... elements) {
        long result = redisCache.hdel(key, elements);
        return result == 1;
    }

    /**
     * 设置单个key value
     * @param key
     * @param element
     * @param value
     * @return
     */
    @Override
    public boolean putMapElement(String key, String element, String value) {
        long result = redisCache.hset(key, element, value);
        return result == 1?true:false;
    }

    /**
     * 判断map中的element是否存在
     * @param key
     * @param element
     * @return
     */
    @Override
    public boolean mapElementExist(String key, String element) {
        return redisCache.hexists(key, element);
    }

    /**
     * 自定义比较器
     */
    class MyComparator implements Comparator<String>{  
          
        @Override  
        public int compare(String o1, String o2) {                
            return o2.compareTo(o1);//降序排列  
        }  
          
    }

    /* (non-Javadoc)
     * 集群单例模式不需要close
     * @see com.ai.ipu.cache.redis.IRedisCache#close()
     */
    @Override
    public void close() throws Exception {
        //redisCache.close();
    }

    @Override
    public Long publish(Object channel, Object message) throws Exception {
        if (channel instanceof String)
        {
            if (StringUtil.isEmpty((String)channel))
                return -2l;
            if (StringUtil.isEmpty((String)message))
                return -3l;
            return redisCache.publish((String)channel, (String)message);
        }
        else if (channel instanceof byte[])
        {
        	if (!ArrayUtils.isEmpty((byte[]) channel))
                if (!ArrayUtils.isEmpty((byte[]) message))
                    return redisCache.publish((byte[])channel, (byte[])message);
                else
                    return -3l;
            else
                return -2l;
        }
        return -1l;
        
    }
    
    @Override
    public void subscribe(AbstractPubSubListener listener, String... channels) throws Exception{
        redisCache.subscribe(listener, channels);
    }

}
